-------------------------------------------------------------------------------
-- |> Config
-- Configure settings and options
-------------------------------------------------------------------------------
-- |> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
-------------------------------------------------------------------------------
-- |> LIBS
-------------------------------------------------------------------------------
local AceConfig = LibStub("AceConfig-3.0")
local SM = LibStub("LibSharedMedia-3.0")
local sounds = SM:List("sound")

-------------------------------------------------------------------------------
-- |> SLASH COMMANDS
-------------------------------------------------------------------------------
local halt = false
--=> Commands Option Table
-------------------------------------------------------------------------------
weizPVP.SlashCommands = {
	name = "Slash Commands",
	order = -3,
	type = "group",
	args = {
		intro = {
			name = "weizPVP Slash Commands",
			type = "description",
			order = 1,
			cmdHidden = true
		},
		config = {
			name = "config",
			desc = "Opens the configuration menu.",
			type = "execute",
			func = function()
				InterfaceOptionsFrame_OpenToCategory("|cffeaeaeaweiz|r|cffffa012PVP|r")
				InterfaceOptionsFrame_OpenToCategory("|cffeaeaeaweiz|r|cffffa012PVP|r")
			end,
			guiHidden = true,
			order = 2
		},
		show = {
			name = "Show",
			desc = "Show the weizPVP window.",
			type = "execute",
			order = 3,
			func = function()
				if not InCombatLockdown() then
					weizPVP.HeaderFrame:Show()
					weizPVP.CoreFrame:Show()
				end
			end,
			dialogHidden = true
		},
		hide = {
			name = "Hide",
			desc = "Hide the weizPVP window.",
			type = "execute",
			order = 4,
			func = function()
				if not InCombatLockdown() then
					weizPVP.HeaderFrame:Hide()
					weizPVP.CoreFrame:Hide()
				end
			end,
			dialogHidden = true
		},
		lock = {
			name = "Lock",
			desc = "Toggle locking of the window's position and size.",
			type = "execute",
			order = 5,
			func = function()
				if not InCombatLockdown() then
					if weizPVP.Options.WindowLocked == true then
						weizPVP:ToggleLocked(false)
					else
						weizPVP:ToggleLocked(true)
					end
				else
					--> In Combat
					weizPVP:PrintAddonMessage("Window can only be locked out of combat.")
				end
			end,
			dialogHidden = true
		},
		pin = {
			name = "Pin",
			desc = "Toggle pinning of the window.",
			type = "execute",
			order = 6,
			func = function()
				if weizPVP.CoreFrame.isLocked then
					weizPVP:PrintAddonMessage("Window is currently |cffff3838LOCKED|r. Can not pin until unlocked.")
				else
					weizPVP:TogglePinned()
				end
				weizPVP:SaveAddonData()
			end,
			dialogHidden = true
		},
		-------------------------------------------------------------------------------
		-- !! HIDDEN COMMANDS ==========================================================
		--> DEBUG: Detects and manages ALL player characters; not just enemies.
		debug = {
			name = "DEBUG",
			desc = "Toggles DEBUG mode where ALL players of any factions are accepted as valid.",
			type = "execute",
			order = 60,
			func = function()
				if weizPVP.Options.GLOBAL_DEBUG == true then
					weizPVP.Options.GLOBAL_DEBUG = false
					weizPVP:ClearListData()
					weizPVP:PrintAddonMessage("|cff37ff37DEBUG DISABLED|r.")
					weizPVP:SetStatusBarMessage("> |cff37ff37DEBUG MODE OFF|r <", true, 1)
				else
					weizPVP.Options.GLOBAL_DEBUG = true
					weizPVP:PrintAddonMessage("|cffff3838DEBUG ENABLED|r.")
					weizPVP:SetStatusBarMessage("> |cffff3838DEBUG MODE ACTIVE|r <", nil, nil, true)
				end
				weizPVP:SaveAddonData()
			end,
			cmdHidden = true
		},
		--> RESETALL: Wipes EVERYTHING; PlayerDB, Options, Current Variables, and then reloads the UI where default settings are applied.
		resetall = {
			name = "Reset All",
			desc = "Resets all settings and wipes player data",
			type = "execute",
			order = 61,
			func = function()
				weizPVP:ResetAll()
			end,
			cmdHidden = true
		},
		-->HALT ALL.PROCESSING
		halt = {
			name = "Halts player updates from Events: Toggles.",
			desc = "Pauses any more events that would gather more player data so we can look at tables at the moment we see a bug",
			type = "execute",
			order = 62,
			func = function()
				if halt == false then
					halt = true
					weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
					weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
					weizPVP:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
					weizPVP:UnregisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
					weizPVP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
					weizPVP:UnregisterEvent("UNIT_HEALTH", "UnitHealthEvent")
					weizPVP:UnregisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
					weizPVP:UnregisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")
					weizPVP:UnregisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")
					weizPVP:UnregisterEvent("DISPLAY_SIZE_CHANGED", "UpdateDisplayData")
					weizPVP:UnregisterEvent("PLAYER_LOGIN", "UpdateDisplayData")
					weizPVP:PrintAddonMessage("|cffff3838RUN: '/wpvp halt' again to resume.|r")
				else
					if halt == true then
						halt = false
						weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
						weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
						weizPVP:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
						weizPVP:RegisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
						weizPVP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
						weizPVP:RegisterEvent("UNIT_HEALTH", "UnitHealthEvent")
						weizPVP:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND", "PlayerEnteringBattlegroundEvent")
						weizPVP:RegisterEvent("PLAYER_ENTERING_WORLD", "PlayerEnteringWorldEvent")
						weizPVP:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChangedNewAreaEvent")
						weizPVP:RegisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
						weizPVP:RegisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")
						weizPVP:RegisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")
						weizPVP:RegisterEvent("DISPLAY_SIZE_CHANGED", "UpdateDisplayData")
						weizPVP:RegisterEvent("PLAYER_LOGIN", "UpdateDisplayData")
						weizPVP:PrintAddonMessage("|cff37ff37HALT ENDED: Events are now being processes again.|r")
					end
				end
			end,
			cmdHidden = true
		},
		--> RESETALL: Wipes EVERYTHING; PlayerDB, Options, Current Variables, and then reloads the UI where default settings are applied.
		resetOptions = {
			name = "Reset Options",
			desc = "Resets all Options but doesn't change player data at all",
			type = "execute",
			order = 63,
			func = function()
				weizPVP:ResetOptions()
			end,
			cmdHidden = true
		}
		-- !!===========================================================================
		-------------------------------------------------------------------------------
	}
}

--=> Register Slash Commands
-------------------------------------------------------------------------------
AceConfig:RegisterOptionsTable("weizPVP.SlashCommands", weizPVP.SlashCommands, {"wpvp", "weizpvp"})
-------------------------------------------------------------------------------
-- |> INTERFACE OPTIONS: WOW GUI MENU
-------------------------------------------------------------------------------
--=> Options Table
-------------------------------------------------------------------------------
weizPVP.OptionsTable = {
	name = "|cffeaeaeaweiz|r|cffffa012PVP|r : v|cff52ecfa" .. weizPVP.Version .. "|r",
	type = "group",
	args = {
		--> General Options Group
		-------------------------------------------
		general = {
			name = "General",
			type = "group",
			order = 1,
			args = {
				introGeneral = {
					name = " |cffffa012General Options|r",
					type = "description",
					fontSize = "large",
					order = 1,
					width = "full"
				},
				--> Addon's General Options
				-------------------------------------------
				lock = {
					name = " Lock Window",
					desc = "Lock the window's location and size.",
					type = "toggle",
					width = "full",
					order = 2,
					get = function(info)
						return weizPVP.Options.WindowLocked
					end,
					set = function(info, value)
						weizPVP:ToggleLocked(value)
					end
				},
				EnableBGs = {
					name = " Enabled in Battlegrounds",
					desc = "Enable weizPVP in Battlegrounds.",
					type = "toggle",
					order = 3,
					width = "full",
					get = function(info)
						return weizPVP.Options.EnabledInBattlegrounds
					end,
					set = function(info, value)
						--> Double check that we were already disabled. If so, initialize addon again.
						weizPVP.Options.EnabledInBattlegrounds = value
						weizPVP.GetPVPZone()
					end
				},
				EnableArenas = {
					name = " Enabled in Arenas",
					desc = "Enable weizPVP in Arenas.",
					type = "toggle",
					order = 4,
					width = "full",
					get = function(info)
						return weizPVP.Options.EnabledInArena
					end,
					set = function(info, value)
						--> Double check that we were already disabled. If so, initialize addon again.
						weizPVP.Options.EnabledInArena = value
						weizPVP.GetPVPZone()
					end
				}
			}
		},
		--> Alerts Group
		-------------------------------------------
		alerts = {
			name = "Alerts",
			type = "group",
			order = 2,
			args = {
				introAlerts = {
					name = " |cffffa012Alerts and Notifications|r",
					type = "description",
					fontSize = "large",
					order = 1,
					width = "full"
				},
				--> New player detected
				-------------------------------------------
				AlertDetectedPlayer = {
					name = " New Player Detected: Audio Alert",
					desc = "Play's an alert when a new player is added to the enemy list. Plays the audio until it ends; doesnt not overlap with multiple detections.",
					type = "toggle",
					width = "full",
					order = 2,
					get = function(info)
						return weizPVP.Options.AudioAlerts.DetectedPlayerSound
					end,
					set = function(info, value)
						weizPVP.Options.AudioAlerts.DetectedPlayerSound = value
					end
				},
				AlertDetectedPlayerSoundFile = {
					type = "select",
					name = " New Player Alert Sound",
					desc = "The audio file that plays on detection of a new player.",
					values = sounds,
					width = "full",
					order = 3,
					get = function()
						for i, v in next, sounds do
							if v == weizPVP.Options.AudioAlerts.DetectedPlayerSoundFile then
								return i
							end
						end
					end,
					set = function(_, value)
						weizPVP.Options.AudioAlerts.DetectedPlayerSoundFile = sounds[value]
					end,
					itemControl = "DDI-Sound"
				},
				--> Stealh
				stealthSection = {
					name = " |cffffa012Stealth Options|r",
					type = "description",
					fontSize = "large",
					width = "full",
					order = 4
				},
				StealthAlertEnabled = {
					name = " Enable Stealth Alerts",
					desc = "Show an alert when a unit is detected going stealth or invis.",
					type = "toggle",
					width = "full",
					order = 5,
					get = function(info)
						return weizPVP.Options.StealthAlert.Enabled
					end,
					set = function(info, value)
						weizPVP.Options.StealthAlert.Enabled = value
					end
				},
				StealthAlertSoundEnabled = {
					name = " Enable Stealth Audio Alert",
					desc = "Plays a sound when players going stealth or invis while Stealth Alerts is enabled.",
					type = "toggle",
					width = "full",
					order = 6,
					get = function(info)
						return weizPVP.Options.StealthAlert.EnableSound
					end,
					set = function(info, value)
						PlaySoundFile(media:Fetch(SOUND, soundList[value]), "Master")
						weizPVP.Options.StealthAlert.EnableSound = value
					end
				},
				StealthAlertSoundFile = {
					type = "select",
					name = " Stealth Alert Sound",
					desc = "The audio file that plays on stealth detection, if enabled.",
					values = sounds,
					width = "full",
					order = 7,
					get = function()
						for i, v in next, sounds do
							if v == weizPVP.Options.StealthAlert.SoundFile then
								return i
							end
						end
					end,
					set = function(_, value)
						weizPVP.Options.StealthAlert.SoundFile = sounds[value]
					end,
					itemControl = "DDI-Sound"
				}
			}
		},
		--> Crosshair Group
		-------------------------------------------
		crosshair = {
			name = "Crosshair",
			type = "group",
			order = 3,
			args = {
				introAlerts = {
					name = " |cffffa012Crosshair Options|r",
					type = "description",
					fontSize = "large",
					order = 1,
					width = "full"
				},
				--> Crosshair Options
				-------------------------------------------
				Enabled = {
					name = " Enable Crosshairs",
					desc = "Toggles the Crosshairs on/off",
					type = "toggle",
					width = "full",
					order = 2,
					get = function(info)
						return weizPVP.Options.Crosshair.Enabled
					end,
					set = function(info, value)
						if value == true then
							weizPVP:EnableCrosshair()
							weizPVP.CrosshairFrame:PLAYER_TARGET_CHANGED()
						else
							weizPVP:DisableCrosshair()
						end
					end
				},
				NameText = {
					name = " Enable Name Text",
					desc = "Shows your target's name on Crosshair overlay.",
					type = "toggle",
					order = 3,
					width = "full",
					get = function(info)
						return weizPVP.Options.Crosshair.NameEnabled
					end,
					set = function(info, value)
						weizPVP.Options.Crosshair.NameEnabled = value
						if weizPVP.Options.Crosshair.Enabled then
							weizPVP.CrosshairFrame:PLAYER_TARGET_CHANGED()
						end
					end
				},
				Guildext = {
					name = " Enable Guild Text",
					desc = "Shows your target's guild name on Crosshair overlay.",
					type = "toggle",
					order = 4,
					width = "full",
					get = function(info)
						return weizPVP.Options.Crosshair.GuildEnabled
					end,
					set = function(info, value)
						weizPVP.Options.Crosshair.GuildEnabled = value
						if weizPVP.Options.Crosshair.Enabled then
							weizPVP.CrosshairFrame:PLAYER_TARGET_CHANGED()
						end
					end
				}
			}
		},
		--> BETA Group
		-------------------------------------------
		beta = {
			name = "BETA",
			type = "group",
			order = 4,
			args = {
				introBeta = {
					name = " |cffffa012BETA Options|r [In-development]",
					type = "description",
					fontSize = "large",
					order = 1,
					width = "full"
				},
				introLagMode = {
					name = " LagMode Options",
					type = "description",
					fontSize = "large",
					order = 2,
					width = "full"
				},
				descLagMode = {
					name = "[Warning]: While LagMode is Active, it disables some detction and info-gathering functions in favor of a lower CPU usage. This may result in player counts that are not 100% correct, or that take longer to detect. \n" ..
						"Health, death, and stealth updates are still processed normally as they are more critical and fast to process.",
					type = "description",
					order = 3,
					width = "full"
				},
				--> LagMode
				-------------------------------------------
				LagModeEnable = {
					name = " LagMode: Enable",
					desc = "Enable LagMode, which will dynamically adjust this addon's cpu usage based on FPS and other factors.",
					type = "toggle",
					order = 4,
					width = "full",
					get = function(info)
						return weizPVP.Options.LagMode.Enabled
					end,
					set = function(info, value)
						weizPVP.Options.LagMode.Enabled = value
					end
				},
				LagModeFPSThreshold = {
					name = "LagMode will run when before this FPS ",
					desc = "LagMode will only run when your FPS is below this value. Deactivates when FPS is above the threshold for a few seconds.",
					type = "range",
					min = 10,
					max = 120,
					width = "full",
					step = 1,
					order = 5,
					get = function(info)
						return weizPVP.Options.LagMode.FPSThreshold
					end,
					set = function(info, value)
						weizPVP.Options.LagMode.FPSThreshold = value
					end
				}
			}
		}
	}
}

LibStub("AceConfig-3.0"):RegisterOptionsTable("weizPVP.OptionsTable", weizPVP.OptionsTable)
LibStub("AceConfigDialog-3.0"):AddToBlizOptions("weizPVP.OptionsTable", "|cffeaeaeaweiz|r|cffffa012PVP|r")
-------------------------------------------------------------------------------
-- |> APPLY OPTIONS
-------------------------------------------------------------------------------
function weizPVP:ApplySavedOptions()
	--> Set window location and dimentions
	weizPVP:SetCoreFramePosition(
		weizPVP.Options.Frames.X,
		weizPVP.Options.Frames.Y,
		weizPVP.Options.Frames.Width,
		weizPVP.Options.Frames.Height
	)
	weizPVP:TogglePinned(weizPVP.Options.WindowPinned)
end
