-------------------------------------------------------------------------------
-- |> Events
-- Manages all events
-------------------------------------------------------------------------------
-- |> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local UnitIsEnemy = UnitIsEnemy
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local GetTime = GetTime
local UnitIsPlayer = UnitIsPlayer
local UnitLevel = UnitLevel
local IsInInstance = IsInInstance
local bit_band = bit.band
local C_Timer_After = C_Timer.After
local InCombatLockdown = InCombatLockdown
local CombatLogGetCurrentEventInfo = CombatLogGetCurrentEventInfo
local UnitGUID = UnitGUID
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local _
local UPDATE_INT = 0.5 --> UNIT_HEALTH UPDATE THROTTLE

-------------------------------------------------------------------------------
-- |> FUNCTIONS
-------------------------------------------------------------------------------
-----------------------------------------------------------------------
--> Static Role Assignment
---------------------------------------------------------------------
function weizPVP:RoleAssign(class)
	if class == "ROGUE" or class == "MAGE" or class == "WARLOCK" or class == "HUNTER" then
		return "DAMAGER"
	else
		return "UNKNOWN"
	end
end

-----------------------------------------------------------------------
--> Get Spec from Spell
-----------------------------------------------------------------------
local specializationID
function weizPVP:GetSpecFromSpell(SpellID)
	specializationID = weizPVP.SpecDB[SpellID]
	return specializationID
end

-----------------------------------------------------------------------
--> Get Role from Spec
-----------------------------------------------------------------------
local roleFromSpec

function weizPVP:GetSpecRole(specid)
	if specid then
		_, _, _, _, roleFromSpec, _, _ = GetSpecializationInfoByID(specid)
		return roleFromSpec or "UNKNOWN"
	else
		return "UNKNOWN"
	end
end

-------------------------------------------------------------------------------------------
-- |> DETERMINE ZONE AND INSTANCE TYPE FROM EVENTS
-------------------------------------------------------------------------------------------
local inInstance
local WarModeEnabled
--> Check for Valid (PVP) Instance
-----------------------------------------------------------------------
function weizPVP:CheckValidInstance()
	--> Checking for Valid BG
	inInstance = select(2, IsInInstance()) --> Get instance info
	if inInstance == "pvp" and weizPVP.Options.EnabledInBattlegrounds == true then
		return true
	end

	if inInstance == "arena" == true and weizPVP.Options.EnabledInArena == true then
		return true
	end
	return false
end

--> Check for Valid World Zone
-----------------------------------------------------------------------
function weizPVP:CheckValidWorldZone()
	--> Checking for Valid WPVP Zone
	inInstance = select(2, IsInInstance())
	WarModeEnabled = UnitIsWarModeDesired("player")
	if inInstance == "none" and WarModeEnabled == true then
		return true
	else
		return false
	end
end

--> Get PVP Zone
-----------------------------------------------------------------------
local ZoneTimerStartTime = GetTime()
local ValidBG
local ValidWorldZone
function weizPVP.GetPVPZone()
	if ZoneTimerStartTime + 1 <= GetTime() then
		ZoneTimerStartTime = GetTime()
		if GetZoneText() == "" then
			C_Timer_After(0.5, weizPVP.GetPVPZone)
			return
		end
		ValidWorldZone = weizPVP:CheckValidWorldZone()
		ValidBG = weizPVP.CheckValidInstance()
		if not InCombatLockdown() then
			if ValidWorldZone or ValidBG then
				weizPVP.OnEnable()
			else
				weizPVP.OnDisable()
			end
		end
	else
		return
	end
end

--=> EVENT: Entering Battleground Instance
-----------------------------------------------------------------------
function weizPVP:PlayerEnteringBattlegroundEvent()
	C_Timer_After(1, weizPVP.GetPVPZone)
	wipe(weizPVP.CurrentNameplates)
end

-----------------------------------------------------------------------
--=> EVENT: ZoneChangedNewAreaEvent
-----------------------------------------------------------------------
function weizPVP:ZoneChangedNewAreaEvent()
	C_Timer_After(1, weizPVP.GetPVPZone)
end

-----------------------------------------------------------------------
--=> EVENT: Player Entering World
-----------------------------------------------------------------------
function weizPVP:PlayerEnteringWorldEvent()
	wipe(weizPVP.CurrentNameplates)
	C_Timer_After(1, weizPVP.GetPVPZone)
end

-----------------------------------------------------------------------
--=> EVENT TIMER; ONE SECOND REPEATING
-----------------------------------------------------------------------
function weizPVP:OneSecondTimer()
	--> FPS Update
	weizPVP.Display.CurrentFPS = GetFramerate()

	--> Check for LagMode
	weizPVP:LagModeCheck()

	--> Manage Timeout Lists
	weizPVP.ManageListTimeouts()
end

--> Role Check
-----------------------------------------------------------------------
local dead
local GUID
local timestamp
local class
local newhealth
local specIdCheck
local _, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, arg12, arg13
-----------------------------------------------------------------------
local function ClRoleCheck(sName, sGuid, spellID)
	-- role = "UNKNOWN"
	class = select(2, GetPlayerInfoByGUID(srcGUID)) or ""

	if class == "ROGUE" or class == "MAGE" or class == "WARLOCK" or class == "HUNTER" then
		return "DAMAGER"
	end

	--> Check Spell ID against DB of specs/roles
	specIdCheck = weizPVP:GetSpecFromSpell(spellID) or nil

	if not specIdCheck then
		return "UNKNOWN"
	end

	--> Role
	role = weizPVP:GetSpecRole(specIdCheck) --> "DAMAGER", "TANK", or "HEALER"

	if role then
		return role
	else
		return "UNKNOWN"
	end
end

-----------------------------------------------------------------------
--=> EVENT PULSE EVENT
-----------------------------------------------------------------------
function weizPVP:PulseEvent()
	--> FPS Update
	weizPVP.Display.CurrentFPS = GetFramerate()

	--> Check for LagMode
	weizPVP:LagModeCheck()

	--> Manage Timeout Lists
	weizPVP.ManageListTimeouts()
end

--> STEALTH CHECK
-----------------------------------------------------------------------
local event
local role3
local stealthFound
local classColor = {1, 1, 1}
local stealthIDs = {1784, 115834, 114018, 11327, 5215, 66, 156136}
-----------------------------------------------------------------------
local function CLStealthCheck(event, arg12, arg13, srcGUID, srcName)
	--> Enabled?
	if weizPVP.Options.StealthAlert.Enabled ~= true then
		return false
	end
	if event ~= "SPELL_AURA_APPLIED" then
		return false
	end

	--> Check for match
	stealthFound = false
	for i = 1, #stealthIDs do
		if stealthIDs[i] == arg12 then
			stealthFound = true
			break
		end
	end

	--> RETURN IF NOT STEALTH
	if stealthFound == false then
		return false
	else
		if weizPVP.PlayerDB[srcName] and weizPVP.PlayerDB[srcName].Class ~= "" then
			class = select(2, GetPlayerInfoByGUID(srcGUID))
		end
		classColor = RAID_CLASS_COLORS[class] or {r = 1, g = 1, b = 1}
		role3 = ClRoleCheck(srcName, srcGUID, arg12)
		if not weizPVP.PlayerDB[srcName] then
			weizPVP:UpdatePlayerDB(srcName, class, nil, nil, nil, true, role3)
		end
		--> Show Stealth Alert
		weizPVP:ShowStealthAlert(arg13 .. " Detected!", srcName, classColor.r, classColor.g, classColor.b)
		return true
	end
end

-----------------------------------------------------------------------
--=> EVENT TIMER; ONE SECOND REPEATING
-----------------------------------------------------------------------
local function ValidLogPlayer(flags, guid, name)
	if
		(bit_band(flags, 0x00000040) ~= 0 or weizPVP.Options.GLOBAL_DEBUG == true) and bit_band(flags, 0x00000400) ~= 0 and
			guid and
			name and
			name ~= "Unknown"
	 then
		return true
	else
		return false
	end
end

-----------------------------------------------------------------------
--> CREATE NEW PLAYER ENTRY IN DB
-----------------------------------------------------------------------
local level
local newRole
local function MakeNewPlayerEntry(event, srcName, srcGUID, arg12, arg13)
	if weizPVP.PlayerDB[srcName] then
		return
	end
	if srcGUID then
		level = weizPVP:GetLevelData(event, srcName, srcGUID, arg12, arg13)
		class = select(2, GetPlayerInfoByGUID(srcGUID))
		newRole = ClRoleCheck(srcName, srcGUID, arg12) or "UNKNOWN"
		weizPVP:AddNewPlayerDB(srcName, class, level, nil, nil, true, newRole)
	end
end

-------------------------------------------------------------------------------------------
-- |> EVENTS: COMBAT LOG
-------------------------------------------------------------------------------------------
local stealth
local role
-----------------------------------------------------------------------
function weizPVP:CombatLogEvent()
	_, event, _, srcGUID, srcName, srcFlags, _, dstGUID, dstName, dstFlags, _, arg12, arg13, _, _, _ =
		CombatLogGetCurrentEventInfo()

	--> IS VALID SOURCE?
	-------------------------
	if ValidLogPlayer(srcFlags, srcGUID, srcName) then

		--!! VIRA DEBUG ------------------------
		if ViragDevTool_AddData then
			ViragDevTool_AddData(
				{
					["srcName"] = srcName,
					["srcFlags"] = srcFlags,
					["arg12"] = arg12,
					["arg13"] = arg13,
					["dstFlags"] = dstFlags,
					["dstName"] = dstName,
				},
				"EventOrFunction: " .. srcName
			)
		end
		--!! VIRA DEBUG ------------------------
		-------------------------

		--> Check if unit is not in DB yet
		-----------------------------------------
		if not weizPVP.PlayerDB[srcName] then
			MakeNewPlayerEntry(event, srcName, srcGUID, arg12, arg13)
		end

		--> Run with lagmode on or off
		-------------------------
		if weizPVP.Options.LagMode.Active == false then
			if
				weizPVP.PlayerDB[srcName].Estimated == true or not weizPVP.PlayerDB[srcName].Role or
					weizPVP.PlayerDB[srcName].Role == "UNKNOWN"
			 then
				weizPVP:CLParseSource(event, srcName, srcGUID, arg12, arg13, srcFlags) --> Parse data we don't have yet
			end
		end

		--> Check stealth and role
		stealth = CLStealthCheck(event, arg12, arg13, srcGUID, srcName)
		role = ClRoleCheck(srcName, srcGUID, arg12)

		--> Update Source
		weizPVP:UpdatePlayerActiveCache(srcName, nil, nil, nil, nil, role, srcGUID)
	end

	--> VALID DST; CHECK DEAD
	-------------------------
	if ValidLogPlayer(dstFlags, dstGUID, dstName) then
		weizPVP:CLDeath(event, dstGUID, dstName)
	end
end

-----------------------------------------------------------------------
--> CL PARSE SOURCE PLAYER
-----------------------------------------------------------------------
local roleTime = 200
local role2
local lastParseList = {}
function weizPVP:CLParseSource(event, srcName, srcGUID, arg12, arg13, srcFlags)
	if weizPVP.PlayerDB[srcName] then
		if not lastParseList[srcGUID] then
			lastParseList[srcGUID] = 0
		end
		if
			weizPVP.PlayerDB[srcName].Role == "UNKNOWN" or
				weizPVP.PlayerDB[srcName].Timestamp + lastParseList[srcGUID] < GetTime()
		 then
			role2 = ClRoleCheck(srcName, srcGUID, arg12)
			if role2 and role2 ~= "UNKNOWN" then
				return role2
			else
				return "UNKNOWN"
			end
		end
	end
end

-------------------------------------------------------------------------------------------
-- |> EVENTS: UNITS
-------------------------------------------------------------------------------------------
--=> UNITS: MOUSEOVER
-----------------------------------------------------------------------
function weizPVP:PlayerMouseoverEvent()
	if not weizPVP.Options.LagMode.Active and UnitExists("mouseover") then
		weizPVP:GetUnitData("mouseover")
	end
end

--=> UNITS: TARGET CHANGED
-- -----------------------------------------------------------------------
function weizPVP:PlayerTargetEvent()
	weizPVP:GetUnitData("target")
	weizPVP:ChangeTargetIcon()
end

--=> UNITS: NAMEPLATE ADDED
-- -----------------------------------------------------------------------
function weizPVP:NameplateAdded(_, unit)
	if (UnitIsEnemy("player", unit) or weizPVP.Options.GLOBAL_DEBUG) and UnitIsPlayer(unit) then
		GUID = UnitGUID(unit)
		--> Update time only
		if weizPVP.PlayerActiveCache[GUID] and weizPVP.PlayerActiveCache[GUID].Estimated == false then
			weizPVP.PlayerActiveCache[GUID].Stealth = false
			weizPVP:AddPlayerDataToNearby(GUID, false)
		else
			weizPVP:GetUnitData(unit)
		end
		weizPVP.CurrentNameplates[GUID] = true
	end
end

-----------------------------------------------------------------------
--=> UNITS: NAMEPLATE REMOVED
-----------------------------------------------------------------------
function weizPVP:NameplateRemoved(_, unit)
	GUID = UnitGUID(unit)
	if weizPVP.PlayerActiveCache[GUID] then
		weizPVP:AddPlayerDataToNearby(GUID, false)
	end
	weizPVP.CurrentNameplates[GUID] = nil
end

-----------------------------------------------------------------------
--=> EVENT: UNIT_HEALTH
-----------------------------------------------------------------------
local lastUpdated = {}
function weizPVP:UnitHealthEvent(_, unit)
	if unit and UnitGUID(unit) then
		GUID = UnitGUID(unit) or nil
		--> Check if player is aleady on bars
		-------------------------------------
		if GUID and weizPVP.PlayersOnBars[GUID] and weizPVP.PlayerActiveCache[GUID] then
			--> Get unit info
			-------------------------------------
			timestamp = GetTime()
			newhealth = UnitHealth(unit) / UnitHealthMax(unit) * 100
			dead = UnitIsDeadOrGhost(unit)

			--> Check if dead status changed; send updates
			-------------------------------------
			if dead ~= weizPVP.PlayerActiveCache[GUID].Dead then
				weizPVP.PlayerActiveCache[GUID].Dead = dead
				weizPVP.PlayerActiveCache[GUID].Health = newhealth
				weizPVP:AddPlayerDataToNearby(GUID, false)
				return
			end

			--> Check if player is aleady on bars
			-------------------------------------
			if newhealth and weizPVP.PlayerActiveCache[GUID].Health ~= newhealth then
				weizPVP.PlayerActiveCache[GUID].Health = newhealth
			end

			--> Update Health is enough time has passed to update the bars
			-------------------------------------
			if not lastUpdated[GUID] or (lastUpdated[GUID] and lastUpdated[GUID] + UPDATE_INT < timestamp) then
				weizPVP:AddPlayerDataToNearby(GUID, false)
			end
		else
			--> Player was not active on bars, but we have their last time listed.
			--> Clear this player's time from the table
			if lastUpdated[GUID] then
				lastUpdated[GUID] = nil
			end
		end
	end
end

-------------------------------------------------------------------------------------------
-- |> EVENTS: COMBAT
-------------------------------------------------------------------------------------------
--=> EVENT: ENTERING COMBAT
-----------------------------------------------------------------------
function weizPVP:EnteringCombat()
	weizPVP:SetStatusBarMessage("|cFFF4564D +++ IN COMBAT +++|r", false, 0)
end

-----------------------------------------------------------------------
--=> EVENT: LEAVING COMBAT
-----------------------------------------------------------------------
function weizPVP:LeavingCombat()
	weizPVP:SetStatusBarMessage("|cFF53F493 --- OUT OF COMBAT ---|r", true, 3)
	weizPVP:ManageListTimeouts()
end

-----------------------------------------------------------------------
--=> EVENT: PLAYER LEVELED UP
-----------------------------------------------------------------------
function weizPVP:PlayerLevelUp()
	weizPVP.Player.Level = UnitLevel("player") --> Update player level
end

--> COMBAT LOG DEATH CHECK
-----------------------------------------------------------------------
function weizPVP:CLDeath(event, dstGUID, dstName)
	if weizPVP.PlayerActiveCache[dstGUID] then
		if event == "UNIT_DIED" or event == "UNIT_DESTROYED" or event == "UNIT_DISSIPATES" or event == "PARTY_KILL" then
			weizPVP.PlayerActiveCache[dstGUID].Dead = true
			weizPVP:UpdatePlayerActiveCache(dstName, nil, nil, nil, true, nil, dstGUID)
			return true
		end
	end
	return false --> return if both src and dst are known
end

--> ENABLE EVENTS
-----------------------------------------------------------------------
function weizPVP.EnableEvents()
	--> DATA COLLECTION
	weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
	weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
	weizPVP:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
	weizPVP:RegisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
	weizPVP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
	weizPVP:RegisterEvent("UNIT_HEALTH", "UnitHealthEvent")

	--> ZONE CHANGED
	weizPVP:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND", "PlayerEnteringBattlegroundEvent")
	weizPVP:RegisterEvent("PLAYER_ENTERING_WORLD", "PlayerEnteringWorldEvent")
	weizPVP:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChangedNewAreaEvent")

	--> COMBAT/PVP
	weizPVP:RegisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
	weizPVP:RegisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")

	--> PLAYER UPDATES
	weizPVP:RegisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")

	--> DISPLAY CHANGES
	weizPVP:RegisterEvent("DISPLAY_SIZE_CHANGED", "UpdateDisplayData")
	weizPVP:RegisterEvent("PLAYER_LOGIN", "UpdateDisplayData")
end

--> DISABLE EVENTS
-----------------------------------------------------------------------
function weizPVP.DisableEvents()
	--> DATA COLLECTION
	weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
	weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
	weizPVP:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
	weizPVP:UnregisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
	weizPVP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
	weizPVP:UnregisterEvent("UNIT_HEALTH", "UnitHealthEvent")

	--> ZONE CHANGED
	-- Zone events do not get disabled. Needed to auto-enable.

	--> COMBAT/PVP
	weizPVP:UnregisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
	weizPVP:UnregisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")

	--> PLAYER UPDATES
	weizPVP:UnregisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")

	--> DISPLAY CHANGES
	weizPVP:UnregisterEvent("DISPLAY_SIZE_CHANGED", "UpdateDisplayData")
	weizPVP:UnregisterEvent("PLAYER_LOGIN", "UpdateDisplayData")
end

-------------------------------------------------------------------------------------------
-- |>   LAGMODE CHECK
-------------------------------------------------------------------------------------------
local LagModeCounter = 0

function weizPVP:LagModeCheck()
	if weizPVP.Options.LagMode.Enabled == true then
		if LagModeCounter >= 4 then
			LagModeCounter = 0
			if weizPVP.Display.CurrentFPS < weizPVP.Options.LagMode.FPSThreshold and weizPVP.NearbyListSize > 12 then
				if weizPVP.Options.LagMode.Active == false then
					--> LAGMODE ON
					weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
					weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
					weizPVP.CurrentNameplates = {}
					weizPVP.Options.LagMode.Active = true
					UPDATE_INT = 1
				end
			elseif weizPVP.Display.CurrentFPS > weizPVP.Options.LagMode.FPSThreshold + 4 then
				if weizPVP.Options.LagMode.Active == true then
					--> LAGMODE OFF
					weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
					weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
					weizPVP.Options.LagMode.Active = false
					UPDATE_INT = 0.5
				end
			end
		else
			LagModeCounter = LagModeCounter + 1
		end
	end
end
