-------------------------------------------------------------------------------
-- |> Player Data Manager
-- Functions that help acquire and manage data involving the PlayerDB
-------------------------------------------------------------------------------
-- |> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local select = select
local UnitIsEnemy = UnitIsEnemy
local UnitIsPlayer = UnitIsPlayer
local GetUnitName = GetUnitName
local GetGuildInfo = GetGuildInfo
local UnitClass = UnitClass
local UnitRace = UnitRace
local UnitLevel = UnitLevel
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local GetTime = GetTime
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local strsub = strsub

-------------------------------------------------------------------------------
-- |> GLOBAL FUNCTIONS
-------------------------------------------------------------------------------
--=> UpdatePlayerActiveCache
-->  Updates data in the PlayerActiveCache and then calls
--> Sends data off the Lists to be processed
local newPlayerOnList
-----------------------------------------------------------------------
function weizPVP:UpdatePlayerActiveCache(name, newDataFound, health, stealth, dead, role, GUID)
	if not weizPVP.PlayerDB[name] then
		weizPVP.PlayerDB[name] = {}
	end
	newPlayerOnList = true
	if weizPVP.PlayerActiveCache[GUID] then
		newPlayerOnList = false
	else
		weizPVP.PlayerActiveCache[GUID] = {}
	end

	--> Update PlayerCache info
	if GUID then
		weizPVP.PlayerActiveCache[GUID].GUID = GUID
		--> NAME
		if name then
			weizPVP.PlayerActiveCache[GUID].Name = name
		elseif weizPVP.PlayerDB[name].Name ~= nil then
			weizPVP.PlayerActiveCache[GUID].Name = weizPVP.PlayerDB[name].Name
		end

		--> LEVEL
		weizPVP.PlayerActiveCache[GUID].Estimated = weizPVP.PlayerDB[name].Estimated
		if weizPVP.PlayerDB[name].Level ~= nil then
			weizPVP.PlayerActiveCache[GUID].Level = weizPVP.PlayerDB[name].Level
		else
			weizPVP.PlayerActiveCache[GUID].Level = 0
			weizPVP.PlayerDB[name].Level = 0 --!!
		end

		--> CLASS
		if weizPVP.PlayerDB[name].Class ~= nil then
			weizPVP.PlayerActiveCache[GUID].Class = weizPVP.PlayerDB[name].Class
		else
			weizPVP.PlayerActiveCache[GUID].Class = "UNKNOWN"
			weizPVP.PlayerDB[name].Class = "UNKNOWN"
		end

		--> GUILD
		if weizPVP.PlayerDB[name].Guild ~= nil then
			weizPVP.PlayerActiveCache[GUID].Guild = weizPVP.PlayerDB[name].Guild
		else
			if weizPVP.PlayerActiveCache[GUID].Estimated == true then
				weizPVP.PlayerActiveCache[GUID].Guild = "?"
				weizPVP.PlayerDB[name].Guild = "?"
			else
				weizPVP.PlayerActiveCache[GUID].Guild = ""
				weizPVP.PlayerDB[name].Guild = ""
			end
		end

		--> HEALTH
		if health ~= nil then
			weizPVP.PlayerActiveCache[GUID].Health = health
		elseif weizPVP.PlayerActiveCache[GUID].Health == nil then
			weizPVP.PlayerActiveCache[GUID].Health = 100
		end

		--> STEALTH
		if stealth ~= nil then
			weizPVP.PlayerActiveCache[GUID].Stealth = stealth
		end

		--> DEAD
		if dead ~= nil then
			weizPVP.PlayerActiveCache[GUID].Dead = dead
			if dead == true then
				weizPVP.PlayerActiveCache[GUID].Health = 0
			end
		end

		--> ROLE
		if role and role ~= "UNKNOWN" then
			weizPVP.PlayerActiveCache[GUID].Role = role
			weizPVP.PlayerDB[name].Role = role
		end
	end

	--> Add player Data to lists
	weizPVP:AddPlayerDataToNearby(GUID, newPlayerOnList)
end

--=> AddNewPlayerDB
--> Adds ONLY NEW Players to DB
-----------------------------------------------------------------------
function weizPVP:AddNewPlayerDB(nameAndRealm, class, level, race, guild, estimated, role)
	if not weizPVP.PlayerDB[nameAndRealm] then
		weizPVP.PlayerDB[nameAndRealm] = {}
	end

	weizPVP.PlayerDB[nameAndRealm].Name = nameAndRealm
	if class then
		weizPVP.PlayerDB[nameAndRealm].Class = class
	end
	weizPVP.PlayerDB[nameAndRealm].Level = level or 0
	weizPVP.PlayerDB[nameAndRealm].Race = race or ""
	weizPVP.PlayerDB[nameAndRealm].Role = role or "UNKNOWN"
	if estimated == true and guild == nil then
		weizPVP.PlayerDB[nameAndRealm].Guild = ""
	else
		weizPVP.PlayerDB[nameAndRealm].Guild = guild
	end
	weizPVP.PlayerDB[nameAndRealm].Timestamp = GetTime()

	weizPVP.PlayerDB[nameAndRealm].Estimated = estimated or true

	return true
end

--=> Update Player DB
--> All new entries and updates to weizPVP.PlayerDB happen here.
local dataAdded
-----------------------------------------------------------------------
function weizPVP:UpdatePlayerDB(nameAndRealm, class, level, race, guild, estimated, role)
	if not weizPVP.PlayerDB[nameAndRealm] then --> NEW PLAYER check
		return
	end

	--> Update player
	if weizPVP.PlayerDB[nameAndRealm].Estimated == true then
		if weizPVP.PlayerDB[nameAndRealm].Level and level and weizPVP.PlayerDB[nameAndRealm].Level < level then
			weizPVP.PlayerDB[nameAndRealm].Level = level
		end

		if estimated == false then
			weizPVP.PlayerDB[nameAndRealm].Estimated = estimated
		end
	else
		--> not estimating, use the level from the db, if higher
		if weizPVP.PlayerDB[nameAndRealm] and level and weizPVP.PlayerDB[nameAndRealm].Level < level then
			weizPVP.PlayerDB[nameAndRealm].Level = level
		end
	end

	--> Get Role
	if role and role ~= "UNKNOWN" then
		weizPVP.PlayerDB[nameAndRealm].Role = role
	end
	--> Get Guild
	if guild ~= nil then
		weizPVP.PlayerDB[nameAndRealm].Guild = guild
	end

	weizPVP.PlayerDB[nameAndRealm].Timestamp = GetTime()

	dataAdded = true
	return dataAdded
end

--=> GET UNIT DATA
-----------------------------------------------------------------------
--> Gets data for events with a UNITID, such as 'target' or 'nameplate'
local Role
local Dead
local Health
local GUID
local timestamp = GetTime()
local name
function weizPVP:GetUnitData(unit)
	--> Confirm Unit is a player that is an enemy or attackable
	if (UnitIsEnemy("player", unit) or weizPVP.Options.GLOBAL_DEBUG) and UnitIsPlayer(unit) then
		info = {}
		timestamp = GetTime()
		name = GetUnitName(unit, true)
		GUID = UnitGUID(unit) or nil
		Role = "UNKNOWN"
		--> Unit has valid name and GUID
		if name and GUID and name ~= "Unknown" then
			--> If info isnt in DB or the DB is edtimated or older, update info
			-- --> Recently saved, unestimated data gets a timestamp bump
			if not weizPVP.PlayerDB[name] then
				weizPVP.PlayerDB[name] = {}
				weizPVP.PlayerDB[name].Name = name
			end
			weizPVP.PlayerDB[name].Estimated = false
			weizPVP.PlayerDB[name].Guild = GetGuildInfo(unit)
			weizPVP.PlayerDB[name].Class = select(2, UnitClass(unit))

			if Role or Role ~= "UNKNOWN" then
				if weizPVP.PlayerDB[name].Role ~= "UNKNOWN" then
					Role = weizPVP.PlayerDB[name].Role
				end
				if weizPVP.PlayerActiveCache[guid] then
					weizPVP.PlayerActiveCache[guid].Role = Role
				end
			end

			weizPVP.PlayerDB[name].Race = select(1, UnitRace(unit))
			weizPVP.PlayerDB[name].Level = UnitLevel(unit)
			weizPVP.PlayerDB[name].Timestamp = timestamp

			Dead = UnitIsDeadOrGhost(unit)
			Health = UnitHealth(unit) / UnitHealthMax(unit) * 100 or nil
			--> Send update
			weizPVP:UpdatePlayerActiveCache(name, true, Health, false, Dead, nil, GUID)
		end
	end
end

--> GET LEVEL DATA
-------------------------------------------------------------------------------
local level
local abilityType
function weizPVP:GetLevelData(event, player, playerGUID, spellId, spellName)
	if player then
		--> Get basic info from GUID
		level = nil
		--> If we should be parsing data
		--> Parse level only for abilities
		abilityType = strsub(event, 1, 5)
		if abilityType == "SWING" or abilityType == "SPELL" or abilityType == "RANGE" then
			level = weizPVP.AbilityLevelDB[spellName]
			if level == nil then
				--> level is nil, so make it a 0 instead
				level = 0
			end
		end
		return level
	end
	return level
end
